@@ -15,14 +15,10 @@
 #define INQUIRY_CMDLEN  6
 #define HEAVY_CHECK_COUNT	10
 
-#define MSG_EMC_CLARIION_UP	"emc_clariion checker report path up"
-#define MSG_EMC_CLARIION_DOWN	"emc_clariion checker report path down"
-#define MSG_EMC_CLARIION_SHAKY	"emc_clariion checker report path is " \
-				"scheduled for shutdown"
-
 struct emc_clariion_checker_context {
 	int run_count;
-	char wwn[64];
+	char wwn[16];
+	unsigned wwn_set;
 };
 
 int emc_clariion(char *devnode, char *msg, void *context)
@@ -37,7 +33,6 @@ int emc_clariion(char *devnode, char *msg, void *context)
 	if (context != NULL) {
 		ctxt = (struct emc_clariion_checker_context *)context;
 		ctxt->run_count++;
-		/* do stuff */
 	}
 
 	if (ctxt->run_count % HEAVY_CHECK_COUNT) {
@@ -58,48 +53,65 @@ int emc_clariion(char *devnode, char *msg, void *context)
 	io_hdr.pack_id = 0;
 	if (ioctl(fd, SG_IO, &io_hdr) < 0) {
 		close (fd);
-		MSG(MSG_EMC_CLARIION_DOWN);
+		MSG("emc_clariion_checker: sending query command failed");
 		return PATH_DOWN;
 	}
 	if (io_hdr.info & SG_INFO_OK_MASK) {
 		close (fd);
-		MSG(MSG_EMC_CLARIION_DOWN);
+		MSG("emc_clariion_checker: query command indicates error");
 		return PATH_DOWN;
 	}
 	close (fd);
 
-	/*
-	 * TODO: Better logging of why a path is considered failed? But
-	 * then, the other checkers don't do that either...
-	 */
-	if (        /* Verify the code page - right page & revision */
-		sense_buffer[1] != 0xc0 || sense_buffer[9] != 0x00
-		/* Effective initiator type */
-		|| sense_buffer[27] != 0x03
-		/* Failover mode should be set to 1 */        
-		|| (sense_buffer[28] & 0x07) != 0x04
-		/* Arraycommpath should be set to 1 */
-		|| (sense_buffer[30] & 0x04) != 0x04
-		/* LUN operations should indicate normal operations */
-		|| sense_buffer[48] != 0x00
-		/* LUN should at least be bound somewhere */
-		|| sense_buffer[4] != 0x00) {
-		MSG(MSG_EMC_CLARIION_DOWN);
+	if (/* Verify the code page - right page & revision */
+	    sense_buffer[1] != 0xc0 || sense_buffer[9] != 0x00) {
+		MSG("emc_clariion_checker: Path unit report page in unknown format");
 		return PATH_DOWN;
 	}
 
+	if ( /* Effective initiator type */
+	    	sense_buffer[27] != 0x03
+		/* Failover mode should be set to 1 */        
+		|| (sense_buffer[28] & 0x07) != 0x04
+		/* Arraycommpath should be set to 1 */
+		|| (sense_buffer[30] & 0x04) != 0x04) {
+		MSG("emc_clariion_checker: Path not correctly configured for failover");
+		return PATH_DOWN;
+	}
+
+	if ( /* LUN operations should indicate normal operations */
+		sense_buffer[48] != 0x00) {
+		MSG("emc_clariion_checker: Path not available for normal operations");
+		return PATH_SHAKY;
+	}
+
+#if 0
+	/* This is not actually an error as the failover to this group
+	 * _would_ bind the path */
+	if ( /* LUN should at least be bound somewhere */
+		sense_buffer[4] != 0x00) {
+		return PATH_UP;
+	}
+#endif	
+	
 	/*
-	 * TODO: If we had a path_checker context per path, I could
 	 * store the LUN WWN there and compare that it indeed did not
 	 * change in between, to protect against the path suddenly
 	 * pointing somewhere else.
 	 */
-	MSG(MSG_EMC_CLARIION_UP);
+
+	if (ctxt->wwn_set) {
+		if (!memcmp(ctxt->wwn, &sense_buffer[10], 16)) {
+			MSG("emc_clariion_checker: Logical Unit WWN has changed!");
+			return PATH_DOWN;
+		}
+	} else {
+		memcpy(ctxt->wwn, &sense_buffer[10], 16);
+		ctxt->wwn_set = 1;
+	}
+	
+	
+	MSG("emc_clariion_checker: Path healthy");
         return PATH_UP;
 
-	/*
-	 * TODO: determine the condition for that
-	 */
-	if (0)
-		MSG(MSG_EMC_CLARIION_SHAKY);
 }
