@@ -32,15 +32,17 @@
 #include <sysfs/dlist.h>
 #include <sysfs/libsysfs.h>
 #include <libdevmapper.h>
+#include <devmapper.h>
 #include <checkers.h>
 #include <path_state.h>
+#include <safe_printf.h>
+#include <callout.h>
 
 #include "main.h"
 #include "devinfo.h"
 #include "config.h"
 #include "pgpolicies.h"
 #include "dict.h"
-#include "callout.h"
 #include "debug.h"
 
 /* helpers */
@@ -202,7 +204,8 @@ devinfo (struct path *curpath)
 			dbg("get uid callout :");
 			dbg("=================");
 			if(safe_sprintf(buff, "%s /block/%s",
-				 hwe->getuid, curpath->dev)) {
+			   hwe->getuid ? hwe->getuid : conf->default_getuid,
+			   curpath->dev)) {
 				fprintf(stderr, "buff too small\n");
 				exit(1);
 			}
@@ -466,7 +469,7 @@ find_mp (char * wwid)
 	struct mpentry * mpe;
 
 	vector_foreach_slot (conf->mptable, mpe, i)
-                if (strcmp(mpe->wwid, wwid) == 0)
+                if (mpe->wwid && strcmp(mpe->wwid, wwid) == 0)
 			return mpe;
 
 	return NULL;
@@ -536,78 +539,6 @@ coalesce_paths (vector mp, vector pathvec)
 	}
 }
 
-static int
-dm_simplecmd (int task, const char *name) {
-	int r = 0;
-	struct dm_task *dmt;
-
-	if (!(dmt = dm_task_create(task)))
-		return 0;
-
-	if (!dm_task_set_name(dmt, name))
-		goto out;
-
-	r = dm_task_run(dmt);
-
-	out:
-		dm_task_destroy(dmt);
-		return r;
-}
-
-static int
-dm_addmap (int task, const char *name, const char *params, unsigned long size) {
-	struct dm_task *dmt;
-
-	if (!(dmt = dm_task_create(task)))
-		return 0;
-
-	if (!dm_task_set_name(dmt, name))
-		goto out;
-
-	if (!dm_task_add_target(dmt, 0, size, DM_TARGET, params))
-		goto out;
-
-	if (dm_task_run(dmt))
-		return 0;
-
-	out:
-	dm_task_destroy(dmt);
-	return 1;
-}
-
-static int
-dm_map_present (char * str)
-{
-        int r = 0;
-	struct dm_task *dmt;
-        struct dm_names *names;
-        unsigned next = 0;
-
-	if (!(dmt = dm_task_create(DM_DEVICE_LIST)))
-		return 0;
-
-	if (!dm_task_run(dmt))
-		goto out;
-
-	if (!(names = dm_task_get_names(dmt)))
-		goto out;
-
-	if (!names->dev) {
-		goto out;
-	}
-
-	do {
-		if (0 == strncmp(names->name, str, strlen(names->name)))
-			r = 1;
-		next = names->next;
-		names = (void *) names + next;
-	} while (next);
-
-	out:
-	dm_task_destroy(dmt);
-	return r;
-}
-
 static int
 dm_get_map(char * name, char * outparams)
 {
@@ -716,7 +647,7 @@ select_selector (struct multipath * mp)
 
 	/* 3) override by LUN specific settings */
 	vector_foreach_slot (conf->mptable, mpe, i) {
-		if (strcmp(mpe->wwid, mp->wwid) == 0) {
+		if (mpe->wwid && strcmp(mpe->wwid, mp->wwid) == 0) {
 			mp->selector = (mpe->selector) ?
 				   mpe->selector : conf->default_selector;
 			mp->selector_args = (mpe->selector_args) ?
@@ -851,7 +782,7 @@ setup_map (vector pathvec, struct multipath * mpp)
 	 * 4) override by LUN specific setting
 	 */
 	vector_foreach_slot (conf->mptable, mpe, i) {
-		if (strcmp(mpe->wwid, mpp->wwid) == 0 &&
+		if (mpe->wwid && strcmp(mpe->wwid, mpp->wwid) == 0 &&
 		    mpe->iopolicy > 0) {
 			iopolicy = mpe->iopolicy;
 			get_pgpolicy_name(iopolicy_name, iopolicy);
@@ -971,7 +902,7 @@ setup_map (vector pathvec, struct multipath * mpp)
 			goto out;
 	}
 
-	if (dm_addmap(op, mapname, mpp->params, mpp->size)) {
+	if (dm_addmap(op, mapname, DM_TARGET, mpp->params, mpp->size)) {
 		dm_simplecmd(DM_DEVICE_REMOVE, mapname);
 		goto out;
 	}
@@ -1155,6 +1086,11 @@ main (int argc, char *argv[])
 	extern char *optarg;
 	extern int optind;
 
+	if (dm_prereq(DM_TARGET, 0, 0, 0)) {
+		dbg("device mapper prerequisites not met");
+		exit(1);
+	}
+
 	/*
 	 * Don't run in parallel
 	 * ie, acquire a F_WRLCK-type lock on RUN
